/*
 * Copyright 2002 Eric Hodel. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. Neither the name of Eric Hodel nor the names of his
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ERIC HODEL AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ERIC HODEL OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* for integration with Composer code */
var gEditor;
var gContentWindow;
var editorShell;

var Editor = function (editorElementId, sourceElementId, deckElementId) {
  this.init(editorElementId, sourceElementId, deckElementId);
}

Editor.prototype = {
  /* view mode constants */
  PreviewMode : 0,
  NormalMode : 1,
  AllTagsMode : 2,
  SourceMode : 3,

  init : function (editorElementId, sourceElementId, deckElementId) {
    this.editorDeck = document.getElementById(deckElementId);
    this.editorElement = document.getElementById(editorElementId);
    this.sourceElement = document.getElementById(sourceElementId);

    this.editorShell = this.editorElement.editorShell;
    this.editorShell.editorType = 'html';
    this.editorShell.webShellWindow = window.window;
    this.editorShell.contentWindow = window._content;

    this.editorShell.RegisterDocumentStateListener(DocumentStateListener);
    this.editorShell.contentsMIMEType = "text/html";
    this.editorShell.LoadUrl("about:blank");

    this.view('normal');

    this.wrapAt = 70;

    editorShell = this.editorShell;

    EditorSharedStartup();
  },

  /*
   * switch view mode
   *
   * Valid modes are 'normal' and 'source'
   */
  view : function(type) {
    var content;

    if (type == 'source' &&
      this.editorShell.GetEditMode() != this.SourceMode) {
      /* format HTML, wrap, wrap at column, encode entities */
      var flags = 2 | 16 | 32 | 256; 
      content = this.editorShell.editor.outputToString("text/html",
        flags, this.wrapAt);

      this.editorShell.SetDisplayMode(this.SourceMode);

      this.sourceElement.value = content;
      this.editorDeck.selectedIndex = 1;
    } else if (type == 'normal' &&
      this.editorShell.GetEditMode() != this.NormalMode) {
      content = this.sourceElement.value
      this.editorShell.SetDisplayMode(this.NormalMode);

      this.editorShell.RebuildDocumentFromSource(content);
      this.editorDeck.selectedIndex = 0;
    }

    this.focus();
  },

  clear : function() {
    this.editorShell.RebuildDocumentFromSource(
      "<html><head></head><body></body></html>");
  },

  /* get the content of the editor */
  get content () {
    /* encode entities */
    var flags = 256; 
    var content = this.editorShell.IsHTMLSourceMode() ?
      this.sourceElement.value :
      this.editorShell.editor.outputToString("text/html", flags);

    content = content.replace(/<!DOCTYPE html PUBLIC "-\/\/W3C\/\/DTD HTML 4.01 Transitional\/\/EN">\s*/, "");
    content = content.replace(/<html>\s*<head>\s*<title><\/title>\s*<\/head>\s*<body>\s*/m, "");

    return content.replace(/<\/body>\s*<\/html>\s*$/, "");
  },

  set content (text) {
    if (this.editorShell.IsHTMLSourceMode())
      this.sourceElement.value = text;
    else
      this.editorShell.RebuildDocumentFromSource(text);
  },

  /* switch the selection's paragraph format */
  set paragraphFormat (format) {
    this.editorShell.SetParagraphFormat(format);
    this.focus();
  },

  focus : function () {
    switch (this.editorShell.GetEditMode()) {
      case this.SourceMode:
        this.sourceElement.focus();
        break;
      default:
      case this.NormalMode:
        this.editorElement.focus();
        break;
    }
  }
};

/*
 * The following are shadowed because we don't need the full
 * functionality from chrome://editor/content/editor.js
 */

var DocumentStateListener = {
  NotifyDocumentCreated : function () { /* borrowed from editor.js */
    gEditor = editorShell.editor;

    if (typeof(DoAllQueryInterfaceOnEditor) == 'function') {
      /* not in mozilla 1.0 */
      DoAllQueryInterfaceOnEditor();
    }

    EditorInitToolbars();

    gContentWindow = window._content;
    gContentWindow.focus();

    // must focus window._content before updating commands
    window.updateCommands("create");

    if (!("InsertCharWindow" in window))
      window.InsertCharWindow = null;
  },

  NotifyDocumentWillBeDestroyed : function () {},

  NotifyDocumentStateChanged : function (isNowDirty) {
    if (isNowDirty) 
      docWasModified = true;

    window.updateCommands("create");
  }
};
